#include "data_rows.h"

#include "par.h"

/*****************************************************************************
 * INTERNAL FUNCTIONS
 *****************************************************************************/

static Status alloc_data_rows(Data_rows **p_d, String name, int dim,
					Bool read_only, Par_info *par_info)
{
    int i, ndim = par_info->ndim, nblocks, nrows_per_block, t;
    int array[MAX_NDIM];
    Data_rows *d;

    MALLOC(d, Data_rows, 1);

    STRING_MALLOC_COPY(d->name, name);

    d->size_info.ndim = ndim;

    INT_MALLOC_COPY(d->size_info.npoints, par_info->npoints, ndim);

    if (par_info->blocked)
    {
	INT_MALLOC_COPY(d->size_info.block_size, par_info->block_size, ndim);
    }
    else
    {
	MALLOC(d->size_info.block_size, int, ndim);

	for (i = 0; i < ndim; i++)
	{
	    if (i < 2)
		d->size_info.block_size[i] = d->size_info.npoints[i];
	    else
		d->size_info.block_size[i] = 1;
	}
    }

    STRING_MALLOC_COPY(d->block_io.name, par_info->file);

    VECTOR_PRODUCT(d->block_io.block_size, d->size_info.block_size, ndim);

    d->block_io.header = par_info->header;
    d->block_io.swapped = par_info->swapped;
    d->block_io.integer = par_info->integer;
    d->block_io.byte_size = par_info->byte_size;
    d->block_io.deflated = FALSE;

    nblocks = BLOCK(d->size_info.npoints[dim], d->size_info.block_size[dim]);
    nrows_per_block = d->block_io.block_size / d->size_info.block_size[dim];

    d->store_info.base_block = -1;
    d->store_info.nblocks = nblocks;
    d->store_info.nrows_per_block = nrows_per_block;

    MALLOC(d->store_info.cum_nblocks, int, ndim);
    MALLOC(d->store_info.cum_block_size, int, ndim);
    MALLOC(d->store_info.cum_nblocks2, int, ndim);
    MALLOC(d->store_info.cum_block_size2, int, ndim);
    MALLOC(d->store_info.cum_npoints2, int, ndim);

    BLOCKS(array, d->size_info.npoints, d->size_info.block_size, ndim);
    CUMULATIVE(d->store_info.cum_nblocks, array, t, ndim);

    d->store_info.delta_block = d->store_info.cum_nblocks[dim];

    if (dim == (ndim-1))
	d->store_info.cum_blocks = t;
    else
	d->store_info.cum_blocks = d->store_info.cum_nblocks[dim+1];

    CUMULATIVE(d->store_info.cum_block_size, d->size_info.block_size, t, ndim);
    d->store_info.delta_row = d->store_info.cum_block_size[dim];

    /* below are CUMULATIVE except skipping dim */

    t = 1;
    for (i = 0; i < ndim; i++)
    {
	d->store_info.cum_nblocks2[i] = t;

	if (i != dim)
	    t *= array[i];
    }

    if (read_only)
    {
	d->store_info.blocks_written = (Bool *) NULL;
    }
    else
    {
	MALLOC(d->store_info.blocks_written, Bool, t);

	for (i = 0; i < t; i++)
	    d->store_info.blocks_written[i] = FALSE;
    }

    t = 1;
    for (i = 0; i < ndim; i++)
    {
	d->store_info.cum_block_size2[i] = t;

	if (i != dim)
	    t *= d->size_info.block_size[i];
    }

    t = 1;
    for (i = 0; i < ndim; i++)
    {
	d->store_info.cum_npoints2[i] = t;

	if (i != dim)
	    t *= d->size_info.npoints[i];
    }

    MALLOC(d->store_info.blocks, float *, nblocks);

    for (i = 0; i < nblocks; i++)
	MALLOC(d->store_info.blocks[i], float, d->block_io.block_size);

    d->dim = dim;

    d->read_only = read_only;
    d->write_only = !d->read_only; /* for now */

    *p_d = d;

    return OK;
}

static Status open_data_file(Data_rows *d, String error_msg)
{
    d->block_io.file = NULL;

    if (d->read_only)
    {
	CHECK_OPEN_FOR_BINARY_READING(d->block_io.file, d->block_io.name);
	CHECK_STATUS(init_block_read(&d->block_io, error_msg));
    }
    else /* might need to upgrade this, assume write_only for now */
    /* but still need to open as write update because of way data accessed */
    {
	CHECK_OPEN_FOR_BINARY_WRITE_UPDATE(d->block_io.file, d->block_io.name);
	CHECK_STATUS(init_block_write(&d->block_io, error_msg));
    }

    return OK;
}

static Status read_data_blocks(Data_rows *d, int base_block, String error_msg)
{
    int i, block;

    d->store_info.base_block = base_block;

    block = base_block;

/*
    printf("in read_data_blocks: base_block = %d\n", base_block);
    printf("in read_data_blocks: last_done = %d\n", d->block_io.last_done);
    printf("in read_data_blocks: block_size = %d\n", d->block_io.block_size);
    printf("in read_data_blocks: byte_size = %d\n", d->block_io.byte_size);
*/

    for (i = 0; i < d->store_info.nblocks; i++)
    {
/*
    printf("in read_data_blocks: i = %d, block = %d\n", i, block);
*/
	CHECK_STATUS(read_file_block(&d->block_io, block, d->store_info.blocks[i], error_msg));

	block += d->store_info.delta_block;
    }

    return OK;
}

static int get_block_written(Data_rows *d, int base_block)
{
    int block[MAX_NDIM], block_written, ndim = d->size_info.ndim;

    ARRAY_OF_INDEX(block, base_block, d->store_info.cum_nblocks, ndim);

    INDEX_OF_ARRAY(block_written, block, d->store_info.cum_nblocks2, ndim);

    return block_written;
}

static Status write_data_blocks(Data_rows *d, int base_block, String error_msg)
{
    int i, block, block_written;

    d->store_info.base_block = base_block;

    block = base_block;

/*
    printf("in write_data_blocks: base_block = %d\n", base_block);
    printf("in write_data_blocks: last_done = %d\n", d->block_io.last_done);
    printf("in write_data_blocks: block_size = %d\n", d->block_io.block_size);
    printf("in write_data_blocks: byte_size = %d\n", d->block_io.byte_size);
*/

    for (i = 0; i < d->store_info.nblocks; i++)
    {
/*
    printf("in write_data_blocks: i = %d, block = %d\n", i, block);
*/
	CHECK_STATUS(write_file_block(&d->block_io, block, d->store_info.blocks[i], error_msg));

	block += d->store_info.delta_block;
    }

    block_written = get_block_written(d, base_block);
    d->store_info.blocks_written[block_written] = TRUE;

    return OK;
}

static void get_block_point(Data_rows *d, int n,
				int *p_base_block, int *p_base_point)
{
    int i, m, base_block, base_point;
    int ndim = d->size_info.ndim;
    int point[MAX_NDIM], block[MAX_NDIM], block_point[MAX_NDIM];

/*
    printf("in get_block_point: n = %d\n", n);
*/

    if (d->in_order)
    {
/*
printf("in order\n");
*/
	ARRAY_OF_INDEX(point, n, d->store_info.cum_npoints2, ndim);

	for (i = 0; i < ndim; i++)
	{
	    block[i] = point[i] / d->size_info.block_size[i];
	    block_point[i] = point[i] % d->size_info.block_size[i];
	}
    }
    else
    {
	m = n / d->store_info.nrows_per_block;
	ARRAY_OF_INDEX(block, m, d->store_info.cum_nblocks2, ndim);

	m = n % d->store_info.nrows_per_block;
	ARRAY_OF_INDEX(block_point, m, d->store_info.cum_block_size2, ndim);
    }

    INDEX_OF_ARRAY(base_block, block, d->store_info.cum_nblocks, ndim);
    INDEX_OF_ARRAY(base_point, block_point, d->store_info.cum_block_size, ndim);
/*
printf("base_block = %d, base_point = %d\n", base_block, base_point);
*/

    *p_base_block = base_block;
    *p_base_point = base_point;
}

/*****************************************************************************
 * EXTERNAL FUNCTIONS
 *****************************************************************************/

Data_rows *new_data_rows(String name, int dim, String mode,
					int in_order, String error_msg)
{
    int nrows;
    Bool read_only;
    Data_rows *d;
    Par_info par_info;

    dim--;

    if (dim < 0)
    {
	sprintf(error_msg, "dim must be > 0");

	return  NULL;
    }

    if (!equal_strings(mode, "r") && !equal_strings(mode, "w"))
    {
	sprintf(error_msg, "mode must be 'r' or 'w'");

	return  NULL;
    }

    if (read_par_file(name, &par_info, error_msg) == ERROR)
	return  NULL;

    if (par_info.deflated)
    {
	sprintf(error_msg, "data cannot be deflated");

	return  NULL;
    }

    if (equal_strings(mode, "r"))
	read_only = TRUE;
    else
	read_only = FALSE;

    if (alloc_data_rows(&d, name, dim, read_only, &par_info) == ERROR)
    {
	sprintf(error_msg, "allocating memory");

	return  NULL;
    }

    if (in_order)
	d->in_order = TRUE;
    else
	d->in_order = FALSE;

    if (dim >= par_info.ndim)
    {
	sprintf(error_msg, "dim must be <= ndim = %d", par_info.ndim);

	delete_data_rows(d);

	return  NULL;
    }

    VECTOR_PRODUCT(nrows, par_info.npoints, par_info.ndim);
    d->row_size = par_info.npoints[dim];
    nrows /= d->row_size;
    d->nrows = nrows;

    if (open_data_file(d, error_msg) == ERROR)
    {
	delete_data_rows(d);

	return  NULL;
    }

    /* need more if deflated ever allowed */
    /* need lots more if Varian ever allowed */

    return d;
}

void delete_data_rows(Data_rows *d)
{
    int i;
    Line error_msg;

    if (!d->read_only && (d->store_info.base_block >= 0))
	(void) write_data_blocks(d, d->store_info.base_block, error_msg);

    FCLOSE(d->block_io.file);

    FREE(d->name, char);

    FREE(d->size_info.npoints, int);
    FREE(d->size_info.block_size, int);

    FREE(d->block_io.name, char);

    FREE(d->store_info.cum_nblocks, int);
    FREE(d->store_info.cum_block_size, int);
    FREE(d->store_info.cum_nblocks2, int);
    FREE(d->store_info.cum_block_size2, int);
    FREE(d->store_info.cum_npoints2, int);

    if (!d->read_only)
	FREE(d->store_info.blocks_written, Bool);

    for (i = 0; i < d->store_info.nblocks; i++)
	FREE(d->store_info.blocks[i], float);

    FREE(d->store_info.blocks, float *);

    FREE(d, Data_rows);
}

int data_rows_length(Data_rows *d)
{
    return d->nrows;
}

int data_rows_index(Data_rows *d, int *point)
{
    int i, m1, m2, n, ndim = d->size_info.ndim;
    int block[MAX_NDIM], block_point[MAX_NDIM];

    if (d->in_order)
    {
	INDEX_OF_ARRAY(n, point, d->store_info.cum_npoints2, ndim);
    }
    else
    {
	for (i = 0; i < ndim; i++)
	{
	    block[i] = point[i] / d->size_info.block_size[i];
	    block_point[i] = point[i] % d->size_info.block_size[i];
	}

	INDEX_OF_ARRAY(m1, block, d->store_info.cum_nblocks2, ndim);
	INDEX_OF_ARRAY(m2, block_point, d->store_info.cum_block_size2, ndim);

	n = m1 * d->store_info.nrows_per_block + m2;
    }

    return n;
}

void data_rows_point(Data_rows *d, int n, int *point)
{
    int i, m, ndim = d->size_info.ndim;
    int block[MAX_NDIM], block_point[MAX_NDIM];

    if (d->in_order)
    {
	ARRAY_OF_INDEX(point, n, d->store_info.cum_npoints2, ndim);
    }
    else
    {
	m = n / d->store_info.nrows_per_block;
	ARRAY_OF_INDEX(block, m, d->store_info.cum_nblocks2, ndim);

	m = n % d->store_info.nrows_per_block;
	ARRAY_OF_INDEX(block_point, m, d->store_info.cum_block_size2, ndim);

	for (i = 0; i < ndim; i++)
	    point[i] = d->size_info.block_size[i] * block[i] + block_point[i];
    }
}

Status data_rows_get_row(Data_rows *d, int n, Set_item_func set_func,
				Generic_ptr data, String error_msg)
{
    int i, j, k, m, p, base_block, base_point;
    int dim = d->dim;

/*
    printf("in data_rows_get_row: n = %d\n", n);
*/

/*  checked in calling code
    if ((n < 0) || (n >= d->nrows))
	RETURN_ERROR_MSG("index out of range");
*/

    get_block_point(d, n, &base_block, &base_point);

    if (base_block != d->store_info.base_block)
	CHECK_STATUS(read_data_blocks(d, base_block, error_msg));

    k = 0;

    for (i = 0; i < d->store_info.nblocks; i++)
    {
	p = base_point;

	m = d->size_info.block_size[dim];

	if (i == (d->store_info.nblocks-1))
	{
	    m = d->size_info.npoints[dim] % d->size_info.block_size[dim];

	    if (m == 0)
		m = d->size_info.block_size[dim];
	}

/*
printf("loop i = %d, p = %d, m = %d\n", i, p, m);
*/
	for (j = 0; j < m; j++)
	{
/*
printf("loop j = %d, p = %d, k = %d\n", j, p, k);
*/
	    (*set_func)(k++, d->store_info.blocks[i][p], data);

	    p += d->store_info.delta_row;
	}
    }

    return OK;
}

Status data_rows_set_row(Data_rows *d, int n, Get_item_func get_func,
				Generic_ptr data, String error_msg)
{
    int i, j, k, m, p, base_block, base_point, block_written;
    int dim = d->dim, old_base_block = d->store_info.base_block;

/*
    printf("in data_rows_set_row: n = %d\n", n);
*/

/*  checked in calling code
    if ((n < 0) || (n >= d->nrows))
	RETURN_ERROR_MSG("index out of range");
*/

    get_block_point(d, n, &base_block, &base_point);

    if (base_block != old_base_block)
    {
	if (old_base_block >= 0)
	    CHECK_STATUS(write_data_blocks(d, old_base_block, error_msg));

	block_written = get_block_written(d, base_block);

	if (d->store_info.blocks_written[block_written])
	{
	    (void) read_data_blocks(d, base_block, error_msg);
	}
	else
	{
	    d->store_info.base_block = base_block;

	    for (i = 0; i < d->store_info.nblocks; i++)
		ZERO_VECTOR(d->store_info.blocks[i], d->block_io.block_size);
	}
    }

    k = 0;

    for (i = 0; i < d->store_info.nblocks; i++)
    {
	p = base_point;

	m = d->size_info.block_size[dim];

	if (i == (d->store_info.nblocks-1))
	{
	    m = d->size_info.npoints[dim] % d->size_info.block_size[dim];

	    if (m == 0)
		m = d->size_info.block_size[dim];
	}

/*
printf("loop i = %d, p = %d, m = %d\n", i, p, m);
*/
	for (j = 0; j < m; j++)
	{
	    d->store_info.blocks[i][p] = (*get_func)(k++, data);
/*
printf("loop j = %d, p = %d, k = %d, v = %f\n", j, p, k-1, d->store_info.blocks[i][p]);
*/

	    p += d->store_info.delta_row;
	}
    }

    return OK;
}
