#ifndef _incl_data_rows
#define _incl_data_rows

#include "macros.h"
#include "types.h"

#include "block_io.h"

typedef float (*Get_item_func)(int n, Generic_ptr data);
typedef void (*Set_item_func)(int n, float x, Generic_ptr data);

#define MAX_NDIM 10

typedef struct
{
    int ndim;
    int *block_size;
    int *npoints;
}   Size_info;

typedef struct
{
    int base_block;
    int delta_block;
    int delta_row;
    int cum_blocks;
    int *cum_nblocks;
    int *cum_block_size;
    int *cum_nblocks2;
    int *cum_block_size2;
    int *cum_npoints2;
    Bool *blocks_written;
    int nblocks;
    int nrows_per_block;
    float **blocks;
}   Store_info;

typedef struct
{
    String name;
    int dim;
    Bool read_only;
    Bool write_only;
    Bool in_order;
    int row_size;
    int nrows;
    Size_info size_info;
    Block_IO block_io;
    Store_info store_info;
}   Data_rows;

extern Data_rows *new_data_rows
	(String name, int dim, String mode, int in_order, String error_msg);

extern void delete_data_rows
	(Data_rows *d);

extern int data_rows_length
	(Data_rows *d);

extern int data_rows_index
	(Data_rows *d, int *point);

extern void data_rows_point
	(Data_rows *d, int n, int *point);

extern Status data_rows_get_row
	(Data_rows *d, int n, Set_item_func set_func,
		Generic_ptr data, String error_msg);

extern Status data_rows_set_row
	(Data_rows *d, int n, Get_item_func get_func,
		Generic_ptr data, String error_msg);

#endif /* _incl_data_rows */
