#include "opus.h"

#include "fft.h"
#include "phase.h"

static int *npoints_in;
static int *npoints_out;
static Bool *have_sample;
static int *sample;
static int **sample_list;
static Bool have_sample2;
static int *sample2;
static int *sample2_list;
static Bool have_sample3;
static int *sample3;
static int *sample3_list;
static int *ignore;
static int **ignore_list;
static float *work_space = NULL;
static float *work_space2 = NULL;
static float *work_space3 = NULL;
static float *work_space4 = NULL;
static float *decay;
static Bool *have_phase;
static Bool *have_decay;
static int *phase_code;
static int *fft_code;
static int dim;

/*
static void print_vec(String msg, float *d, int n)
{
    int i, j, k;

    printf("%s %d\n", msg, n);
    for (i = 0; i < n/8; i++)
    {
	for (j = 0; j < 8; j++)
	{
	    k = 8*i + j;
    	    if (k >= n)
		break;
	    printf(" %7.4e", d[k]);
	}
	printf("\n");
    }
}
*/

static void do_expand(int n1, int n2, int *list, float *d1, float *d2)
{
    int i, j, k;

    if (n1 == n2)
    {
	if (d1 != d2)
	    COPY_VECTOR(d1, d2, n1);

	return;
    }

/* 31 May 2007: add zeroing here instead of below */
/* 20 Mar 2008: this required more care because could have d1 = d2 */
/* so added work_space2, work_space3 and work_space4 to play safe */
    ZERO_VECTOR(d1, n1);

    for (i = n2-1; i >= 0; i--)
	d1[list[i]] = d2[i];

/* 31 May 2007: no longer assume list in ascending order
    for (i = 0; i < n2; i++)
    {
	j = list[i] + 1;

	if (i == (n2-1))
	    k = n2 - j;  31 May 2007: this should have been n1 - j
	else
	    k = list[i+1] - j;

	ZERO_VECTOR(d1+j, k);
    }

    ZERO_VECTOR(d1, list[0]);
*/

/*
    print_vec("expand d2", d2, n2);
    print_vec("expand d1", d1, n1);
*/
}

static void do_contract(int n1, int n2, int *list, int m, int ntimes,
							float *d1, float *d2)
{
    int i, j, n;

    if (n1 == n2)
    {
	if (d1 != d2)
	{
	    n = n1 * ntimes;
	    COPY_VECTOR(d1, d2, n);
	}

	return;
    }

    for (i = 0; i < ntimes; i++)
    {
	for (j = 0; j < n1; j++)
	    d1[j] = d2[j%m+m*list[j/m]];

	d1 += n1;
	d2 += n2;
    }

/*
    print_vec("contract d2", d2, n2);
    print_vec("contract d1", d1, n1);
*/
}

static void ignore1(float *data)
{
    if (ignore[0] > 0)
	do_contract(npoints_in[0], npoints_in[0]+ignore[0],
					ignore_list[0], 1, 1, data, data);
}

static void ignore2(float *data)
{
    int n;

    n = npoints_in[1] + ignore[1];

    if (ignore[0] > 0)
    	do_contract(npoints_in[0], npoints_in[0]+ignore[0],
					ignore_list[0], 1, n, data, data);

    if (ignore[1] > 0)
    	do_contract(npoints_in[0]*npoints_in[1], npoints_in[0]*n,
				ignore_list[1], npoints_in[0], 1, data, data);
}

static void ignore3(float *data)
{
    int m, n;

    n = npoints_in[1] + ignore[1];
    n *= npoints_in[2] + ignore[2];
    if (ignore[0] > 0)
    	do_contract(npoints_in[0], npoints_in[0]+ignore[0],
					ignore_list[0], 1, n, data, data);

    m = npoints_in[1] + ignore[1];
    n = npoints_in[2] + ignore[2];
    if (ignore[1] > 0)
    	do_contract(npoints_in[0]*npoints_in[1], npoints_in[0]*m,
				ignore_list[1], npoints_in[0], n, data, data);

    m = npoints_in[0] * npoints_in[1];
    if (ignore[2] > 0)
	do_contract(m*npoints_in[2], m*n, ignore_list[2], m, 1, data, data);
}

static void opus(float *data_out, float *data_in)
{
    int i, n;
    float d;

    for (i = npoints_out[dim]-1; i >= 0; i--)	/* real part of image */
	work_space[2*i] = data_in[i];

    for (i = 0; i < npoints_out[dim]; i++)	/* imag part of image = 0 */
	work_space[2*i+1] = 0;

    if (have_phase[dim])
	do_phase(phase_code[dim]+1, work_space);

    do_fft_process(fft_code[dim]+1, work_space);

    if (have_sample2)
    	n = sample2[dim];
    else if (have_sample3)
    	n = sample3[dim];
    else
    	n = sample[dim];

    if (have_decay[dim])
    {
	d = decay[dim];

	for (i = 2; i < n; i += 2)
	{
	    work_space[i] *= d;
	    work_space[i+1] *= d;
            d *= decay[dim];
	}
    }

    if (have_sample2 || have_sample3)
    {
	if (work_space != data_out)
	    COPY_VECTOR(data_out, work_space, n);
    }
    else
    {
	do_contract(npoints_in[dim], n, sample_list[dim], 1, 1,
						data_out, work_space);
    }
}

static void tropus(float *data_out, float *data_in)
{
    int i, n;
    float d;

/* do not want any problems in do_expand if data_in = work_space */
/* so introduced work_space4 here 20 Mar 2008 */

    if (have_sample2)
    {
	n = sample2[dim];
	COPY_VECTOR(work_space4, data_in,  n);
    }
    else if (have_sample3)
    {
	n = sample3[dim];
	COPY_VECTOR(work_space4, data_in,  n);
    }
    else
    {
	n = sample[dim];
	do_expand(n, npoints_in[dim], sample_list[dim], work_space4, data_in);
    }

    if (have_decay[dim])
    {
	d = decay[dim];

	for (i = 2; i < n; i += 2)
	{
	    work_space4[i] *= d;
	    work_space4[i+1] *= d;
	    d *= decay[dim];
	}
    }

    ZERO_VECTOR(work_space4+n, 2*npoints_out[dim]-n);
    do_fft_process(fft_code[dim], work_space4);

    if (have_phase[dim])
	do_phase(phase_code[dim], work_space4);

    for (i = 0; i < npoints_out[dim]; i++)
	data_out[i] = work_space4[2*i];
}

static void opus1(float *data_out, float *data_in)
{
    dim = 0;
    opus(data_out, data_in);
}

static void tropus1(float *data_out, float *data_in)
{
    dim = 0;
    tropus(data_out, data_in);
}

static void get_1d_from_2d(float *data, int n1, int n2)
{
    int i;

    for (i = 0; i < n2; i++)
	work_space[i] = data[i*n1];
}

/* added work as argument 20 Mar 2008 */
static void put_1d_into_2d(float *data, float *work, int n1, int n2)
{
    int i;

    for (i = 0; i < n2; i++)
	data[i*n1] = work[i];
}

static void opus2(float *data_out, float *data_in)
{
    int i, nin, nout, n0, n1;

/*
    print_vec("opus2A", data_in, 128);
*/
    if (have_sample2)
    {
	n0 = sample2[0];
	n1 = sample2[1];
    }
    else
    {
	n0 = npoints_in[0];
	n1 = npoints_in[1];
    }

    dim = 0;
    for (i = 0; i < npoints_out[1]; i++)
	opus(data_out+i*n0, data_in+i*npoints_out[0]);

    dim = 1;
    for (i = 0; i < n0; i++)
    {
	get_1d_from_2d(data_out+i, n0, npoints_out[1]);
	opus(work_space, work_space);
	put_1d_into_2d(data_out+i, work_space, n0, n1);
    }

    if (have_sample2)
    {
	nin = n0 * n1;
	nout = npoints_in[0] * npoints_in[1];
/*
	do_contract(nout, nin, sample2_list, 1, 1, data_out, data_out);
*/
/* 25 Feb 2007: put in so that can have list not be increasing */
	do_contract(nout, nin, sample2_list, 1, 1, work_space2, data_out);
	COPY_VECTOR(data_out, work_space2, nout);
    }
/*
    print_vec("opus2B", data_out, 128);
*/
}

static void tropus2(float *data_out, float *data_in)
{
    int i, nin, nout, n0, n1;

/*
    print_vec("tropus2A", data_in, 128);
*/
    if (have_sample2)
    {
	nin = npoints_in[0] * npoints_in[1];
	nout = sample2[0] * sample2[1];
	do_expand(nout, nin, sample2_list, work_space2, data_in);
	data_in = work_space2;
	n0 = sample2[0];
	n1 = sample2[1];
    }
    else
    {
	n0 = npoints_in[0];
	n1 = npoints_in[1];
    }

    dim = 0;
    for (i = n1-1; i >= 0; i--)
	tropus(data_out+i*npoints_out[0], data_in+i*n0);

    dim = 1;
    for (i = 0; i < npoints_out[0]; i++)
    {
	get_1d_from_2d(data_out+i, npoints_out[0], n1);
	tropus(work_space2, work_space);
	put_1d_into_2d(data_out+i, work_space2, npoints_out[0], npoints_out[1]);
    }
/*
    print_vec("tropus2B", data_out, 128);
*/
}

static void opus3(float *data_out, float *data_in)
{
    int i, j, kin, kout, nin, nout, n0, n1, n2;

    if (have_sample3)
    {
	n0 = sample3[0];
	n1 = sample3[1];
	n2 = sample3[2];
    }
    else
    {
	n0 = npoints_in[0];
	n1 = npoints_in[1];
	n2 = npoints_in[2];
    }

    dim = 0;
    for (i = 0; i < npoints_out[1]*npoints_out[2]; i++)
	opus(data_out+i*n0, data_in+i*npoints_out[0]);

    dim = 1;
    for (i = 0; i < npoints_out[2]; i++)
    {
	kin = i * n0 * npoints_out[1];
	kout = i * n0 * n1;

	for (j = 0; j < n0; j++)
	{
	    get_1d_from_2d(data_out+kin+j, n0, npoints_out[1]);
	    opus(work_space, work_space);
	    put_1d_into_2d(data_out+kout+j, work_space, n0, n1);
	}
    }

    dim = 2;
    for (i = 0; i < n0*n1; i++)
    {
	get_1d_from_2d(data_out+i, n0*n1, npoints_out[2]);
	opus(work_space, work_space);
	put_1d_into_2d(data_out+i, work_space, n0*n1, n2);
    }

    if (have_sample3)
    {
	nin = n0 * n1 * n2;
	nout = npoints_in[0] * npoints_in[1] * npoints_in[2];
	do_contract(nout, nin, sample3_list, 1, 1, data_out, data_out);
    }
}

static void tropus3(float *data_out, float *data_in)
{
    int i, j, kin, kout, nin, nout, n0, n1, n2;

    if (have_sample3)
    {
	nin = npoints_in[0] * npoints_in[1] * npoints_in[2];
	nout = sample2[0] * sample2[1] * sample2[2];
	do_expand(nout, nin, sample3_list, work_space3, data_in);
	data_in = work_space3;
	n0 = sample3[0];
	n1 = sample3[1];
	n2 = sample3[2];
    }
    else
    {
	n0 = npoints_in[0];
	n1 = npoints_in[1];
	n2 = npoints_in[2];
    }

    dim = 0;
    for (i = n1*n2-1; i >= 0; i--)
	tropus(data_out+i*npoints_out[0], data_in+i*n0);

    dim = 1;
    for (i = n2-1; i >= 0; i--)
    {
	kin = i * npoints_out[0] * n1;
	kout = i * npoints_out[0] * npoints_out[1];

	for (j = npoints_out[0]-1; j >= 0; j--)
	{
	    get_1d_from_2d(data_out+kin+j, npoints_out[0], n1);
	    tropus(work_space3, work_space);
	    put_1d_into_2d(data_out+kout+j, work_space3, npoints_out[0], npoints_out[1]);
	}
    }

    dim = 2;
    for (i = 0; i < npoints_out[0]*npoints_out[1]; i++)
    {
	get_1d_from_2d(data_out+i, npoints_out[0]*npoints_out[1],
							n2);
	tropus(work_space3, work_space);
	put_1d_into_2d(data_out+i, work_space3, npoints_out[0]*npoints_out[1],
							npoints_out[2]);
    }
}

static Status check_opus_alloc(int n)
{
/*  could use store->row_data for ndim = 1, but hardly a major issue  */

    MALLOC(work_space, float, n);
/* added work_space4 20 Mar 2008 */
    MALLOC(work_space4, float, n);

    if (have_sample2)
    {
	MALLOC(work_space2, float, MAX(n, sample2[0]*sample2[1]));
    }
    else
    {
/* added this case 20 Mar 2008 */
/* (do not really need this and work_space3 below but just easier) */
	MALLOC(work_space2, float, n);
    }

    if (have_sample3)
    {
	MALLOC(work_space3, float, MAX(n, sample3[0]*sample3[1]*sample3[2]));
    }
    else
    {
/* added this case 20 Mar 2008 */
/* (do not really need this and work_space2 above but just easier) */
	MALLOC(work_space3, float, n);
    }

    return  OK;
}

Status init_opus(int ndim, int *npts_in, int *npts_out,
		int *ign, int **ign_list, Ignore_func *ign_func,
		Bool *have_smp, int *smp, int **smp_list,
		Bool have_smp2, int *smp2, int *smp2_list,
		Bool have_smp3, int *smp3, int *smp3_list,
		int *fft_cd, Bool *have_phs, int *phs_cd,
		Bool *have_dcy, float *dcy,
		Transform *op, Transform *trop, String error_msg)
{
    int i, n = 0;

    if ((ndim < 1) || (ndim > 3))  /* this should never happen */
	RETURN_ERROR_MSG("'maxent': number of dims. must be 1, 2, or 3");

    npoints_in = npts_in;
    npoints_out = npts_out;
    ignore = ign;
    ignore_list = ign_list;
    have_sample = have_smp;
    sample = smp;
    sample_list = smp_list;
    have_sample2 = have_smp2;
    sample2 = smp2;
    sample2_list = smp2_list;
    have_sample3 = have_smp3;
    sample3 = smp3;
    sample3_list = smp3_list;
    fft_code = fft_cd;
    have_phase = have_phs;
    phase_code = phs_cd;
    have_decay = have_dcy;
    decay = dcy;

    for (i = 0; i < ndim; i++)
	n = MAX(n, npoints_out[i]);

    if (check_opus_alloc(2*n) == ERROR)
	RETURN_ERROR_MSG("'maxent': allocating memory");

    if (ndim == 1)
    {
	*op = opus1;
	*trop = tropus1;
	*ign_func = ignore1;
    }
    else if (ndim == 2)
    {
	*op = opus2;
	*trop = tropus2;
	*ign_func = ignore2;
    }
    else if (ndim == 3)
    {
	*op = opus3;
	*trop = tropus3;
	*ign_func = ignore3;
    }

    return  OK;
}
