#include "par.h"
#include "utility.h"

#define  MAX_NDIM  10
#define  UCSF_FILE_HEADER  180
#define  UCSF_DIM_HEADER  128
#define  UCSF_MAX_HEADER (UCSF_FILE_HEADER + MAX_NDIM*UCSF_DIM_HEADER)

#define  POINTS_READ  8192

static void set_int(int *x, int y, Bool big)
{
    *x = y;

    if (!big)
	swap_bytes((char *) x, BYTES_PER_WORD);
}

static void set_float(float *x, float y, Bool big)
{
    *x = y;

    if (!big)
	swap_bytes((char *) x, BYTES_PER_WORD);
}

int main(int argc, char **argv)
{
    int i, j, k, n, ndim, npoints[MAX_NDIM], block_size[MAX_NDIM], m = POINTS_READ;
    Bool big = is_big_endian();
    float p, x[POINTS_READ];
    char h[UCSF_MAX_HEADER];
    Ref_info *ref[MAX_NDIM];
    Line file_in, file_out, error_msg;
    Par_info par_info;
    FILE *fpr, *fpw;

    if (argc != 3)
    {
	sprintf(error_msg,
	    "correct syntax: azara2ucsf <input par file> <output UCSF file>");
	ERROR_AND_EXIT(error_msg);
    }

    strcpy(file_in, argv[1]);
    strcpy(file_out, argv[2]);

/*
    printf("Par file: ");
    scanf("%s", file_in);
*/

    if (read_par_file(file_in, &par_info, error_msg) == ERROR)
	ERROR_AND_EXIT(error_msg);

    if (par_info.deflated)
	ERROR_AND_EXIT("cannot convert deflated data");

    ndim = par_info.ndim;

    for (i = 0; i < ndim; i++)
    {
	j = ndim - i - 1; /* UCSF does dims backwards */
	npoints[i] = par_info.npoints[j];
	ref[i] = par_info.ref + j;

	if (par_info.blocked)
	    block_size[i] = par_info.block_size[j];
	else if (j == 0)
	    block_size[i] = par_info.npoints[j];
	else
	    block_size[i] = 1;
    }

    if ((fpr = fopen(par_info.file, READ)) == NULL)
	ERROR_AND_EXIT("opening data file");

/*
    printf("UCSF file: ");
    scanf("%s", file_out);
*/

    if ((fpw = fopen(file_out, WRITE)) == NULL)
	ERROR_AND_EXIT("opening UCSF file");

    ZERO_VECTOR(h, UCSF_MAX_HEADER);

    strcpy(h, "UCSF NMR");

    h[10] = (char) ndim;
    h[11] = (char) 1; /* real data */
    h[13] = (char) 2; /* UCSF format version number */

    for (i = 0; i < ndim; i++)
    {
	j = UCSF_FILE_HEADER + i*UCSF_DIM_HEADER;

	strcpy(h+j, ref[i]->nuc); 
	h[j+5] = 0; /* make sure null terminated */
	set_int((int *) (h+j+8), npoints[i], big);
	set_int((int *) (h+j+16), block_size[i], big);
	set_float((float *) (h+j+20), ref[i]->sf, big);
	set_float((float *) (h+j+24), ref[i]->sw, big);
	p = 0.5 * ((float) npoints[i]); /* center point */
	convert_from_point(REF_PPM, npoints[i], ref[i], &p);
	set_float((float *) (h+j+28), p, big);
    }

    n = UCSF_FILE_HEADER + ndim*UCSF_DIM_HEADER;

    if (fwrite((void *) h, 1, n, fpw) != n)
	ERROR_AND_EXIT("writing UCSF header");

    k = fread((void *) x, BYTES_PER_WORD, m, fpr);

    while (k > 0)
    {
	if (big && par_info.swapped)
	    swap_bytes((char *) x, BYTES_PER_WORD*k);
	else if (!big && !par_info.swapped)
	    swap_bytes((char *) x, BYTES_PER_WORD*k);

	if (par_info.integer)
	    float_words(x, k);

	if (fwrite((void *) x, BYTES_PER_WORD, k, fpw) != k)
	    ERROR_AND_EXIT("writing UCSF data");

	k = fread((void *) x, BYTES_PER_WORD, m, fpr);
    }

    fclose(fpr);
    fclose(fpw);

    return 0;
}
