#!/usr/local/bin/python

import os
import string
import sys

def fatal_error(error_msg):

  print error_msg
  sys.exit(1)

def find_ndim(par_file):

  fp = open(par_file)

  lines = fp.readlines()

  fp.close()

  for line in lines:

    n = string.find(line, '!') # remove comments
    if (n >= 0):
      line = line[:n]

    line = string.strip(line) # remove white space

    if (line[:4] == 'ndim'):
      return string.atoi(string.strip(line[4:]))

  fatal_error('could not find ndim')

def do_transpose(ndim, par_file, transposed_file):

  script_file = 'project.scr'

  fp = open(script_file, 'w')

  fp.write('input ' + par_file + '\n')
  fp.write('output ' + transposed_file + '\n')
  fp.write('project')

  for i in range(ndim):
    fp.write(' ' + str(ndim-i))

  fp.write('\n')
  fp.close()

  cmd = 'project ' + script_file
  os.system(cmd)

  os.remove(script_file)

def convert_to_ucsf(transposed_file, ucsf_file):

  par_file = transposed_file + '.par'

  cmd = 'azara2ucsf ' + par_file + ' ' + ucsf_file
  os.system(cmd)

  os.remove(transposed_file)
  os.remove(par_file)
  
def main():

  if (len(sys.argv) != 2):

    fatal_error('correct syntax: ' + sys.argv[0] + ' <par file>')

  par_file = sys.argv[1]

  ndim = find_ndim(par_file)
  print 'ndim = ' + str(ndim)

  n = string.find(par_file, '.')

  if (n < 0):
    n = len(par_file)

  ucsf_file = par_file[:n] + '.ucsf'
  print 'UCSF file = ' + ucsf_file

  transposed_file = 'transposed_file.spc'
  do_transpose(ndim, par_file, transposed_file)

  convert_to_ucsf(transposed_file, ucsf_file)

main()
